@@ -20,17 +20,15 @@
 
 #include "kbaloorolesprovider.h"
 
-#include <QDebug>
-#include <KLocalizedString>
-
 #include <Baloo/File>
 #include <KFileMetaData/PropertyInfo>
 #include <KFileMetaData/UserMetaData>
 #include <KFormat>
+#include <KLocalizedString>
 
-#include <QTime>
-#include <QMap>
 #include <QCollator>
+#include <QDebug>
+#include <QTime>
 
 struct KBalooRolesProviderSingleton
 {
@@ -148,6 +146,7 @@ KBalooRolesProvider::KBalooRolesProvider() :
         { "lineCount",     "lineCount" },
         { "width",         "imageSize" },
         { "height",        "imageSize" },
+        { "imageDateTime",   "imageDateTime"},
         { "nexif.orientation", "orientation", },
         { "artist",     "artist" },
         { "genre",	"genre"  },
