@@ -20,7 +20,7 @@
 
 #include "behaviorsettingspage.h"
 
-#include "dolphin_generalsettings.h"
+#include "views/viewproperties.h"
 
 #include <KLocalizedString>
 
@@ -29,8 +29,6 @@
 #include <QRadioButton>
 #include <QVBoxLayout>
 
-#include <views/viewproperties.h>
-
 BehaviorSettingsPage::BehaviorSettingsPage(const QUrl& url, QWidget* parent) :
     SettingsPageBase(parent),
     m_url(url),
