@@ -22,16 +22,15 @@
 
 #include "spaceinfoobserver.h"
 
-#include <QMouseEvent>
-
 #include <KLocalizedString>
 #include <KNS3/KMoreToolsMenuFactory>
-#include <KIO/Job>
 #include <knewstuff_version.h>
 
+#include <QMouseEvent>
+
 StatusBarSpaceInfo::StatusBarSpaceInfo(QWidget* parent) :
     KCapacityBar(KCapacityBar::DrawTextInline, parent),
-    m_observer(0)
+    m_observer(nullptr)
 {
     setCursor(Qt::PointingHandCursor);
 }
